=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2006-01-31 07:08:09 +0000
+++ bzrlib/bzrdir.py	2006-01-31 11:51:27 +0000
@@ -40,13 +40,13 @@
     BzrDir instances let you create or open any of the things that can be
     found within .bzr - checkouts, branches and repositories.
     
-    base
-        base directory this is located under.
+    transport
+        the transport which this bzr dir is rooted at (i.e. file:///.../.bzr/)
     """
 
     @staticmethod
     def create(base):
-        """Create a new BzrDir at the url 'bzr'.
+        """Create a new BzrDir at the url 'base'.
         
         This will call the current default formats initialize with base
         as the only parameter.
@@ -56,6 +56,22 @@
         """
         return BzrDirFormat.get_default_format().initialize(safe_unicode(base))
 
+    @staticmethod
+    def create_repository(base):
+        """Create a new BzrDir and Repository at the url 'base'.
+
+        This will use the current default BzrDirFormat, and use whatever 
+        repository format that that uses for bzrdirformat.create_repository.
+
+        The Repository object is returned.
+
+        This must be overridden as an instance method in child classes, where
+        it should take no parameters and construct whatever repository format
+        that child class desires.
+        """
+        bzrdir = BzrDir.create(base)
+        return bzrdir.create_repository()
+
     def __init__(self, _transport, _format):
         """Initialize a Bzr control dir object.
         
@@ -66,7 +82,7 @@
         _transport: the transport this dir is based at.
         """
         self._format = _format
-        self._transport = _transport
+        self.transport = _transport
 
     @staticmethod
     def open_unsupported(base):
@@ -118,6 +134,56 @@
                 raise errors.NotBranchError(path=url)
             t = new_t
 
+    def open_repository(self):
+        """Open the repository object at this BzrDir if one is present.
+        
+        TODO: static convenience version of this?
+        TODO: NoRepositoryError that can be raised.
+        """
+        raise NotImplementedError(self.open_repository)
+
+
+class BzrDir4(BzrDir):
+    """A .bzr version 4 control object."""
+
+    def create_repository(self):
+        """See BzrDir.create_repository."""
+        from bzrlib.repository import RepositoryFormat4
+        return RepositoryFormat4().initialize(self)
+
+    def open_repository(self):
+        """See BzrDir.open_repository."""
+        from bzrlib.repository import RepositoryFormat4
+        return RepositoryFormat4().open(self, _found=True)
+
+
+class BzrDir5(BzrDir):
+    """A .bzr version 5 control object."""
+
+    def create_repository(self):
+        """See BzrDir.create_repository."""
+        from bzrlib.repository import RepositoryFormat5
+        return RepositoryFormat5().initialize(self)
+
+    def open_repository(self):
+        """See BzrDir.open_repository."""
+        from bzrlib.repository import RepositoryFormat5
+        return RepositoryFormat5().open(self, _found=True)
+
+
+class BzrDir6(BzrDir):
+    """A .bzr version 6 control object."""
+
+    def create_repository(self):
+        """See BzrDir.create_repository."""
+        from bzrlib.repository import RepositoryFormat6
+        return RepositoryFormat6().initialize(self)
+
+    def open_repository(self):
+        """See BzrDir.open_repository."""
+        from bzrlib.repository import RepositoryFormat6
+        return RepositoryFormat6().open(self, _found=True)
+
 
 class BzrDirFormat(object):
     """An encapsulation of the initialization and open routines for a format.
@@ -137,7 +203,7 @@
     """
 
     _default_format = None
-    """The default format used for new branches."""
+    """The default format used for new .bzr dirs."""
 
     _formats = {}
     """The known formats."""
@@ -191,7 +257,7 @@
                 control_files.put_utf8(file, content)
         finally:
             control_files.unlock()
-        return BzrDir(t, self)
+        return self.open(t, _found=True)
 
     def is_supported(self):
         """Is this format supported?
@@ -210,7 +276,15 @@
         if not _found:
             assert isinstance(BzrDirFormat.find_format(transport),
                               self.__class__)
-        return BzrDir(transport, self)
+        return self._open(transport)
+
+    def _open(self, transport):
+        """Template method helper for opening BzrDirectories.
+
+        This performs the actual open and any additional logic or parameter
+        passing.
+        """
+        raise NotImplementedError(self._open)
 
     @classmethod
     def register_format(klass, format):
@@ -255,6 +329,10 @@
         """
         return False
 
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        return BzrDir4(transport, self)
+
 
 class BzrDirFormat5(BzrDirFormat):
     """Bzr control format 5.
@@ -270,6 +348,10 @@
         """See BzrDirFormat.get_format_string()."""
         return "Bazaar-NG branch, format 5\n"
 
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        return BzrDir5(transport, self)
+
 
 class BzrDirFormat6(BzrDirFormat):
     """Bzr control format 6.
@@ -285,6 +367,10 @@
         """See BzrDirFormat.get_format_string()."""
         return "Bazaar-NG branch, format 6\n"
 
+    def _open(self, transport):
+        """See BzrDirFormat._open."""
+        return BzrDir6(transport, self)
+
 
 BzrDirFormat.register_format(BzrDirFormat4())
 BzrDirFormat.register_format(BzrDirFormat5())

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2006-01-26 20:52:58 +0000
+++ bzrlib/repository.py	2006-01-31 11:51:27 +0000
@@ -14,8 +14,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from copy import deepcopy
 from cStringIO import StringIO
+from unittest import TestSuite
 
+import bzrlib.bzrdir as bzrdir
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.errors import InvalidRevisionId
 from bzrlib.lockable_files import LockableFiles
@@ -24,6 +27,7 @@
 from bzrlib.store import copy_all
 from bzrlib.store.weave import WeaveStore
 from bzrlib.store.text import TextStore
+from bzrlib.trace import mutter
 from bzrlib.tree import RevisionTree
 from bzrlib.testament import Testament
 from bzrlib.tree import EmptyTree
@@ -43,25 +47,31 @@
     remote) disk.
     """
 
-    def __init__(self, transport, branch_format):
-        # circular dependencies:
-        from bzrlib.branch import (BzrBranchFormat4,
-                                   BzrBranchFormat5,
-                                   BzrBranchFormat6,
-                                   )
+    @staticmethod
+    def create(a_bzrdir):
+        """Construct the current default format repository in a_bzrdir."""
+        return RepositoryFormat.get_default_format().initialize(a_bzrdir)
+
+    def __init__(self, transport, branch_format, _format=None, a_bzrdir=None):
         object.__init__(self)
-        self.control_files = LockableFiles(transport.clone(bzrlib.BZRDIR), 'README')
+        if transport is not None:
+            self.control_files = LockableFiles(transport.clone(bzrlib.BZRDIR), 'README')
+        else: 
+            # TODO: clone into repository if needed
+            self.control_files = LockableFiles(a_bzrdir.transport, 'README')
 
         dir_mode = self.control_files._dir_mode
         file_mode = self.control_files._file_mode
+        self._format = _format
+        self.bzrdir = a_bzrdir
 
         def get_weave(name, prefixed=False):
             if name:
-                name = bzrlib.BZRDIR + '/' + safe_unicode(name)
+                name = safe_unicode(name)
             else:
-                name = bzrlib.BZRDIR
+                name = ''
             relpath = self.control_files._escape(name)
-            weave_transport = transport.clone(relpath)
+            weave_transport = self.control_files._transport.clone(relpath)
             ws = WeaveStore(weave_transport, prefixed=prefixed,
                             dir_mode=dir_mode,
                             file_mode=file_mode)
@@ -76,11 +86,11 @@
             # some existing branches where there's a mixture; we probably 
             # still want the option to look for both.
             if name:
-                name = bzrlib.BZRDIR + '/' + safe_unicode(name)
+                name = safe_unicode(name)
             else:
-                name = bzrlib.BZRDIR
+                name = ''
             relpath = self.control_files._escape(name)
-            store = TextStore(transport.clone(relpath),
+            store = TextStore(self.control_files._transport.clone(relpath),
                               prefixed=prefixed, compressed=compressed,
                               dir_mode=dir_mode,
                               file_mode=file_mode)
@@ -90,16 +100,29 @@
             #    store = bzrlib.store.CachedStore(store, cache_path)
             return store
 
+        if branch_format is not None:
+            # circular dependencies:
+            from bzrlib.branch import (BzrBranchFormat4,
+                                       BzrBranchFormat5,
+                                       BzrBranchFormat6,
+                                       )
+            if isinstance(branch_format, BzrBranchFormat4):
+                self._format = RepositoryFormat4()
+            elif isinstance(branch_format, BzrBranchFormat5):
+                self._format = RepositoryFormat5()
+            elif isinstance(branch_format, BzrBranchFormat6):
+                self._format = RepositoryFormat6()
+            
 
-        if isinstance(branch_format, BzrBranchFormat4):
+        if isinstance(self._format, RepositoryFormat4):
             self.inventory_store = get_store('inventory-store')
             self.text_store = get_store('text-store')
             self.revision_store = get_store('revision-store')
-        elif isinstance(branch_format, BzrBranchFormat5):
+        elif isinstance(self._format, RepositoryFormat5):
             self.control_weaves = get_weave('')
             self.weave_store = get_weave('weaves')
             self.revision_store = get_store('revision-store', compressed=False)
-        elif isinstance(branch_format, BzrBranchFormat6):
+        elif isinstance(self._format, RepositoryFormat6):
             self.control_weaves = get_weave('')
             self.weave_store = get_weave('weaves', prefixed=True)
             self.revision_store = get_store('revision-store', compressed=False,
@@ -112,6 +135,16 @@
     def lock_read(self):
         self.control_files.lock_read()
 
+    @staticmethod
+    def open(base):
+        """Open the repository rooted at base.
+
+        For instance, if the repository is at URL/.bzr/repository,
+        Repository.open(URL) -> a Repository instance.
+        """
+        control = bzrdir.BzrDir.open(base)
+        return control.open_repository()
+
     def unlock(self):
         self.control_files.unlock()
 
@@ -280,3 +313,208 @@
     def sign_revision(self, revision_id, gpg_strategy):
         plaintext = Testament.from_revision(self, revision_id).as_short_text()
         self.store_revision_signature(gpg_strategy, plaintext, revision_id)
+
+
+class RepositoryFormat(object):
+    """A repository format.
+
+    Formats provide three things:
+     * An initialization routine to construct repository data on disk.
+     * a format string which is used when the BzrDir supports versioned
+       children.
+     * an open routine which returns a Repository instance.
+
+    Formats are placed in an dict by their format string for reference 
+    during opening. These should be subclasses of RepositoryFormat
+    for consistency.
+
+    Once a format is deprecated, just deprecate the initialize and open
+    methods on the format class. Do not deprecate the object, as the 
+    object will be created every system load.
+
+    Common instance attributes:
+    _matchingbzrdir - the bzrdir format that the repository format was
+    originally written to work with. This can be used if manually
+    constructing a bzrdir and repository, or more commonly for test suite
+    parameterisation.
+    """
+
+    _default_format = None
+    """The default format used for new branches."""
+
+    _formats = {}
+    """The known formats."""
+
+    @classmethod
+    def get_default_format(klass):
+        """Return the current default format."""
+        return klass._default_format
+
+    def get_format_string(self):
+        """Return the ASCII format string that identifies this format.
+        
+        Note that in pre format ?? repositories the format string is 
+        not permitted nor written to disk.
+        """
+        raise NotImplementedError(self.get_format_string)
+
+    def initialize(self, a_bzrdir):
+        """Create a weave repository.
+        
+        TODO: when creating split out bzr branch formats, move this to a common
+        base for Format5, Format6. or something like that.
+        """
+        from bzrlib.weavefile import write_weave_v5
+        from bzrlib.weave import Weave
+        
+        # Create an empty weave
+        sio = StringIO()
+        bzrlib.weavefile.write_weave_v5(Weave(), sio)
+        empty_weave = sio.getvalue()
+
+        mutter('creating repository in %s.', a_bzrdir.transport.base)
+        dirs = ['revision-store', 'weaves']
+        lock_file = 'branch-lock'
+        files = [('inventory.weave', StringIO(empty_weave)), 
+                 ]
+        
+        # FIXME: RBC 20060125 dont peek under the covers
+        # NB: no need to escape relative paths that are url safe.
+        control_files = LockableFiles(a_bzrdir.transport, 'branch-lock')
+        control_files.lock_write()
+        control_files._transport.mkdir_multi(dirs,
+                mode=control_files._dir_mode)
+        try:
+            for file, content in files:
+                control_files.put(file, content)
+        finally:
+            control_files.unlock()
+        return Repository(None, branch_format=None, _format=self, a_bzrdir=a_bzrdir)
+
+    def is_supported(self):
+        """Is this format supported?
+
+        Supported formats must be initializable and openable.
+        Unsupported formats may not support initialization or committing or 
+        some other features depending on the reason for not being supported.
+        """
+        return True
+
+    def open(self, a_bzrdir, _found=False):
+        """Return an instance of this format for the bzrdir a_bzrdir.
+        
+        _found is a private parameter, do not use it.
+        """
+        if not _found:
+            raise NotImplementedError
+        return Repository(None, branch_format=None, _format=self, a_bzrdir=a_bzrdir)
+
+    @classmethod
+    def register_format(klass, format):
+        klass._formats[format.get_format_string()] = format
+
+    @classmethod
+    def set_default_format(klass, format):
+        klass._default_format = format
+
+    @classmethod
+    def unregister_format(klass, format):
+        assert klass._formats[format.get_format_string()] is format
+        del klass._formats[format.get_format_string()]
+
+
+class RepositoryFormat4(RepositoryFormat):
+    """Bzr repository format 4.
+
+    This repository format has:
+     - flat stores
+     - TextStores for texts, inventories,revisions.
+
+    This format is deprecated: it indexes texts using a text id which is
+    removed in format 5; initializationa and write support for this format
+    has been removed.
+    """
+
+    def __init__(self):
+        super(RepositoryFormat4, self).__init__()
+        self._matchingbzrdir = bzrdir.BzrDirFormat4()
+
+    def initialize(self, url):
+        """Format 4 branches cannot be created."""
+        raise errors.UninitializableFormat(self)
+
+    def is_supported(self):
+        """Format 4 is not supported.
+
+        It is not supported because the model changed from 4 to 5 and the
+        conversion logic is expensive - so doing it on the fly was not 
+        feasible.
+        """
+        return False
+
+
+class RepositoryFormat5(RepositoryFormat):
+    """Bzr control format 5.
+
+    This repository format has:
+     - weaves for file texts and inventory
+     - flat stores
+     - TextStores for revisions and signatures.
+    """
+
+    def __init__(self):
+        super(RepositoryFormat5, self).__init__()
+        self._matchingbzrdir = bzrdir.BzrDirFormat5()
+
+
+class RepositoryFormat6(RepositoryFormat):
+    """Bzr control format 6.
+
+    This repository format has:
+     - weaves for file texts and inventory
+     - hash subdirectory based stores.
+     - TextStores for revisions and signatures.
+    """
+
+    def __init__(self):
+        super(RepositoryFormat6, self).__init__()
+        self._matchingbzrdir = bzrdir.BzrDirFormat6()
+
+# formats which have no format string are not discoverable
+# and not independently creatable, so are not registered.
+# __default_format = RepositoryFormatXXX()
+# RepositoryFormat.register_format(__default_format)
+# RepositoryFormat.set_default_format(__default_format)
+_legacy_formats = [RepositoryFormat4(),
+                   RepositoryFormat5(),
+                   RepositoryFormat6()]
+
+
+class RepositoryTestProviderAdapter(object):
+    """A tool to generate a suite testing multiple repository formats at once.
+
+    This is done by copying the test once for each transport and injecting
+    the transport_server, transport_readonly_server, and bzrdir_format and
+    repository_format classes into each copy. Each copy is also given a new id()
+    to make it easy to identify.
+    """
+
+    def __init__(self, transport_server, transport_readonly_server, formats):
+        self._transport_server = transport_server
+        self._transport_readonly_server = transport_readonly_server
+        self._formats = formats
+    
+    def adapt(self, test):
+        result = TestSuite()
+        for repository_format, bzrdir_format in self._formats:
+            new_test = deepcopy(test)
+            new_test.transport_server = self._transport_server
+            new_test.transport_readonly_server = self._transport_readonly_server
+            new_test.bzrdir_format = bzrdir_format
+            new_test.repository_format = repository_format
+            def make_new_test_id():
+                new_id = "%s(%s)" % (new_test.id(), repository_format.__class__.__name__)
+                return lambda: new_id
+            new_test.id = make_new_test_id()
+            result.addTest(new_test)
+        return result

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-01-31 07:08:09 +0000
+++ bzrlib/tests/__init__.py	2006-01-31 11:51:27 +0000
@@ -82,11 +82,13 @@
     import bzrlib.tests.blackbox
     import bzrlib.tests.branch_implementations
     import bzrlib.tests.bzrdir_implementations
+    import bzrlib.tests.repository_implementations
     return [
             bzrlib.doc,
             bzrlib.tests.blackbox,
             bzrlib.tests.branch_implementations,
             bzrlib.tests.bzrdir_implementations,
+            bzrlib.tests.repository_implementations,
             ]
 
 
@@ -864,6 +866,7 @@
                    'bzrlib.tests.test_parent',
                    'bzrlib.tests.test_permissions',
                    'bzrlib.tests.test_plugins',
+                   'bzrlib.tests.test_repository',
                    'bzrlib.tests.test_revision',
                    'bzrlib.tests.test_revisionnamespaces',
                    'bzrlib.tests.test_revprops',

=== modified file 'bzrlib/tests/bzrdir_implementations/test_bzrdir.py'
--- bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-01-31 07:08:09 +0000
+++ bzrlib/tests/bzrdir_implementations/test_bzrdir.py	2006-01-31 11:51:27 +0000
@@ -30,9 +30,7 @@
                            UninitializableFormat,
                            NotBranchError,
                            )
-import bzrlib.gpg
-from bzrlib.osutils import getcwd
-from bzrlib.revision import NULL_REVISION
+import bzrlib.repository as repository
 from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
 from bzrlib.trace import mutter
 import bzrlib.transactions as transactions
@@ -87,6 +85,8 @@
         # storage is made
         # create a standalone_branch , clone that and check all bits are
         # clone.
+        # these should all check that the formats on subthings like repository
+        # are the same as the source format.
         # features that are not supported (like detached working trees for
         # older formats, should catch the error and silently pass the test 
         # as they are honouring the format.
@@ -99,7 +99,6 @@
         # have checkouts force creation of a new branch because thats the 
         # desired semantic.
 
-
     def test_format_initialize_find_open(self):
         # loopback test to check the current format initializes to itself.
         if not self.bzrdir_format.is_supported():
@@ -127,3 +126,29 @@
         self.assertRaises(NotBranchError,
                           self.bzrdir_format.open,
                           get_transport(self.get_readonly_url()))
+
+    def test_create_repository(self):
+        # a bzrdir can construct a repository for itself.
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        t = get_transport(self.get_url())
+        made_control = self.bzrdir_format.initialize(t.base)
+        made_repo = made_control.create_repository()
+        self.failUnless(isinstance(made_repo, repository.Repository))
+        self.assertEqual(made_control, made_repo.bzrdir)
+        
+    def test_open_repository(self):
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        t = get_transport(self.get_url())
+        made_control = self.bzrdir_format.initialize(t.base)
+        made_repo = made_control.create_repository()
+        opened_repo = made_control.open_repository()
+        self.assertEqual(made_control, opened_repo.bzrdir)
+        self.failUnless(isinstance(opened_repo, made_repo.__class__))

=== added directory 'bzrlib/tests/repository_implementations'
=== added file 'bzrlib/tests/repository_implementations/__init__.py'
--- bzrlib/tests/repository_implementations/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/repository_implementations/__init__.py	2006-01-31 11:51:27 +0000
@@ -0,0 +1,54 @@
+# Copyright (C) 2006 by Canonical Ltd
+# Authors: Robert Collins <robert.collins@canonical.com>
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""BzrDir implementation tests for bzr.
+
+These test the conformance of all the repository variations to the expected API.
+Specific tests for individual formats are in the tests/test_repository.py file 
+rather than in tests/branch_implementations/*.py.
+"""
+
+from bzrlib.repository import (_legacy_formats,
+                               RepositoryFormat,
+                               RepositoryTestProviderAdapter,
+                               )
+                            
+from bzrlib.tests import (
+                          adapt_modules,
+                          default_transport,
+                          TestLoader,
+                          TestSuite,
+                          )
+
+
+def test_suite():
+    result = TestSuite()
+    test_repository_implementations = [
+        'bzrlib.tests.repository_implementations.test_repository',
+        ]
+    adapter = RepositoryTestProviderAdapter(
+        default_transport,
+        # None here will cause a readonly decorator to be created
+        # by the TestCaseWithTransport.get_readonly_transport method.
+        None,
+        [(format, format._matchingbzrdir) for format in 
+         RepositoryFormat._formats.values() + _legacy_formats])
+    loader = TestLoader()
+    adapt_modules(test_repository_implementations, adapter, loader, result)
+    return result

=== added file 'bzrlib/tests/repository_implementations/test_repository.py'
--- bzrlib/tests/repository_implementations/test_repository.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/repository_implementations/test_repository.py	2006-01-31 11:51:27 +0000
@@ -0,0 +1,108 @@
+# (C) 2005, 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for bzrdir implementations - tests a bzrdir format."""
+
+import os
+import sys
+
+import bzrlib.branch as branch
+import bzrlib.bzrdir as bzrdir
+from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
+from bzrlib.commit import commit
+import bzrlib.errors as errors
+from bzrlib.errors import (FileExists,
+                           NoSuchRevision,
+                           NoSuchFile,
+                           UninitializableFormat,
+                           NotBranchError,
+                           )
+import bzrlib.repository as repository
+from bzrlib.tests import TestCase, TestCaseWithTransport, TestSkipped
+from bzrlib.tests.bzrdir_implementations.test_bzrdir import TestCaseWithBzrDir
+from bzrlib.trace import mutter
+import bzrlib.transactions as transactions
+from bzrlib.transport import get_transport
+from bzrlib.upgrade import upgrade
+from bzrlib.workingtree import WorkingTree
+
+
+class TestCaseWithRepository(TestCaseWithBzrDir):
+
+    def setUp(self):
+        super(TestCaseWithRepository, self).setUp()
+
+    def make_repository(self, relpath):
+        bzrdir = self.make_bzrdir(relpath)
+        return bzrdir.create_repository()
+
+
+class TestRepository(TestCaseWithRepository):
+
+    def test_clone(self):
+        #TODO: Test that cloning a repository preserves all the information
+        # such as signatures etc etc.
+        raise AssertionError('not tested yet.')
+
+    def test_format_initialize_find_open(self):
+        # loopback test to check the current format initializes to itself.
+        if not self.repository_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        # supported formats must be able to init and open
+        t = get_transport(self.get_url())
+        readonly_t = get_transport(self.get_readonly_url())
+        made_control = self.bzrdir_format.initialize(t.base)
+        made_repo = self.repository_format.initialize(made_control)
+        self.failUnless(isinstance(made_repo, repository.Repository))
+        self.assertEqual(made_control, made_repo.bzrdir)
+
+        # find it via bzrdir opening:
+        opened_control = bzrdir.BzrDir.open(readonly_t.base)
+        direct_opened_repo = opened_control.open_repository()
+        self.assertEqual(direct_opened_repo.__class__, made_repo.__class__)
+        self.assertEqual(opened_control, direct_opened_repo.bzrdir)
+
+        self.failUnless(isinstance(direct_opened_repo._format,
+                        self.repository_format.__class__))
+        # find it via Repository.open
+        opened_repo = repository.Repository.open(readonly_t.base)
+        self.failUnless(isinstance(opened_repo, made_repo.__class__))
+        self.assertEqual(made_repo._format.__class__,
+                         opened_repo._format.__class__)
+        # if it has a unique id string, can we probe for it ?
+        try:
+            self.repository_format.get_format_string()
+        except NotImplementedError:
+            return
+        self.assertEqual(self.repository_format,
+                         repository.RepositoryFormat.find_format(opened_control))
+
+
+    def test_create_repository(self):
+        # a repository can be constructedzrdir can construct a repository for itself.
+        if not self.bzrdir_format.is_supported():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            return
+        t = get_transport(self.get_url())
+        made_control = self.bzrdir_format.initialize(t.base)
+        made_repo = made_control.create_repository()
+        self.failUnless(isinstance(made_repo, repository.Repository))
+        self.assertEqual(made_control, made_repo.bzrdir)

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2006-01-31 07:08:09 +0000
+++ bzrlib/tests/test_bzrdir.py	2006-01-31 11:51:27 +0000
@@ -43,12 +43,20 @@
         # creating a bzr dir should now create an instrumented dir.
         try:
             result = bzrdir.BzrDir.create('memory:/')
-            self.assertEqual(result,  'A bzr dir')
+            self.failUnless(isinstance(result, SampleBzrDir))
         finally:
             bzrdir.BzrDirFormat.set_default_format(old_format)
         self.assertEqual(old_format, bzrdir.BzrDirFormat.get_default_format())
 
 
+class SampleBzrDir(bzrdir.BzrDir):
+    """A sample BzrDir implementation to allow testing static methods."""
+
+    def create_repository(self):
+        """See BzrDir.create_repository."""
+        return "A repository"
+
+
 class SampleBzrDirFormat(bzrdir.BzrDirFormat):
     """A sample format
 
@@ -65,7 +73,7 @@
         t = get_transport(url)
         t.mkdir('.bzr')
         t.put('.bzr/branch-format', StringIO(self.get_format_string()))
-        return 'A bzr dir'
+        return SampleBzrDir(t, self)
 
     def is_supported(self):
         return False
@@ -121,6 +129,16 @@
         # now open_downlevel should fail too.
         self.assertRaises(UnknownFormatError, bzrdir.BzrDir.open_unsupported, url)
 
+    def test_create_repository(self):
+        format = SampleBzrDirFormat()
+        old_format = bzrdir.BzrDirFormat.get_default_format()
+        bzrdir.BzrDirFormat.set_default_format(format)
+        try:
+            repo = bzrdir.BzrDir.create_repository(self.get_url())
+            self.assertEqual('A repository', repo)
+        finally:
+            bzrdir.BzrDirFormat.set_default_format(old_format)
+        
 
 class ChrootedTests(TestCaseWithTransport):
     """A support class that provides readonly urls outside the local namespace.

=== added file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_repository.py	2006-01-31 11:51:27 +0000
@@ -0,0 +1,79 @@
+# (C) 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the Repository facility that are not interface tests.
+
+For interface tests see tests/repository_implementations/*.py.
+
+For concrete class tests see this file, and for storage formats tests
+also see this file.
+"""
+
+from StringIO import StringIO
+
+import bzrlib.bzrdir as bzrdir
+from bzrlib.errors import (NotBranchError,
+                           UnknownFormatError,
+                           UnsupportedFormatError,
+                           )
+import bzrlib.repository as repository
+from bzrlib.tests import TestCase, TestCaseWithTransport
+from bzrlib.transport import get_transport
+from bzrlib.transport.http import HttpServer
+from bzrlib.transport.memory import MemoryServer
+
+
+class TestDefaultFormat(TestCase):
+
+    def test_get_set_default_initializer(self):
+        old_format = repository.RepositoryFormat.get_default_format()
+        # default is None - we cannot create a Repository independently yet
+        self.assertEqual(old_format, None)
+        repository.RepositoryFormat.set_default_format(SampleRepositoryFormat())
+        # creating a repository should now create an instrumented dir.
+        try:
+            # directly
+            control = bzrdir.BzrDir.create('memory:/')
+            result = repository.Repository.create(control)
+            self.assertEqual(result, 'A bzr repository dir')
+        finally:
+            repository.RepositoryFormat.set_default_format(old_format)
+        self.assertEqual(old_format, repository.RepositoryFormat.get_default_format())
+
+
+class SampleRepositoryFormat(repository.RepositoryFormat):
+    """A sample format
+
+    this format is initializable, unsupported to aid in testing the 
+    open and open_downlevel routines.
+    """
+
+    def get_format_string(self):
+        """See RepositoryFormat.get_format_string()."""
+        return "Sample .bzr repository format."
+
+    def initialize(self, a_bzrdir):
+        """Initialize a repository in a BzrDir"""
+        t = a_bzrdir.transport
+        t.mkdir('repository')
+        t.put('repository/format', StringIO(self.get_format_string()))
+        return 'A bzr repository dir'
+
+    def is_supported(self):
+        return False
+
+    def open(self, a_bzrdir):
+        return "opened repository."

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2006-01-31 07:08:09 +0000
+++ bzrlib/tests/test_selftest.py	2006-01-31 11:51:27 +0000
@@ -254,6 +254,32 @@
         self.assertEqual(tests[1].transport_readonly_server, server2)
 
 
+class TestRepositoryProviderAdapter(TestCase):
+    """A group of tests that test the repository implementation test adapter."""
+
+    def test_adapted_tests(self):
+        # check that constructor parameters are passed through to the adapted
+        # test.
+        from bzrlib.repository import RepositoryTestProviderAdapter
+        input_test = TestRepositoryProviderAdapter(
+            "test_adapted_tests")
+        server1 = "a"
+        server2 = "b"
+        formats = [("c", "C"), ("d", "D")]
+        adapter = RepositoryTestProviderAdapter(server1, server2, formats)
+        suite = adapter.adapt(input_test)
+        tests = list(iter(suite))
+        self.assertEqual(2, len(tests))
+        self.assertEqual(tests[0].bzrdir_format, formats[0][1])
+        self.assertEqual(tests[0].repository_format, formats[0][0])
+        self.assertEqual(tests[0].transport_server, server1)
+        self.assertEqual(tests[0].transport_readonly_server, server2)
+        self.assertEqual(tests[1].bzrdir_format, formats[1][1])
+        self.assertEqual(tests[1].repository_format, formats[1][0])
+        self.assertEqual(tests[1].transport_server, server1)
+        self.assertEqual(tests[1].transport_readonly_server, server2)
+
+
 class TestTestCaseWithTransport(TestCaseWithTransport):
     """Tests for the convenience functions TestCaseWithTransport introduces."""
 

